# Generated by Django 4.1.5 on 2024-06-07 20:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0020_cloudcredentials_cloudcredentialsmetadata'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cloudcredentials',
            name='group',
            field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='cloud_credentials', to='account.group'),
        ),
    ]
