# Generated by Django 4.1.3 on 2022-12-14 18:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('location', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='geoip',
            name='subnet',
            field=models.CharField(db_index=True, default=None, max_length=64, null=True),
        ),
        migrations.AlterField(
            model_name='geoip',
            name='ip',
            field=models.CharField(db_index=True, max_length=64),
        ),
    ]
