# Generated by Django 4.1.3 on 2022-12-20 04:13

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('account', '0012_settings_settingsmetadata'),
    ]

    operations = [
        migrations.CreateModel(
            name='Metrics',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('start', models.DateTimeField(db_index=True)),
                ('granularity', models.CharField(db_index=True, max_length=64)),
                ('uuid', models.SlugField(max_length=124, unique=True)),
                ('slug', models.SlugField(max_length=124)),
                ('k1', models.SlugField(default=None, max_length=64, null=True)),
                ('v1', models.IntegerField(default=0)),
                ('k2', models.SlugField(default=None, max_length=64, null=True)),
                ('v2', models.IntegerField(default=0)),
                ('k3', models.SlugField(default=None, max_length=64, null=True)),
                ('v3', models.IntegerField(default=0)),
                ('k4', models.SlugField(default=None, max_length=64, null=True)),
                ('v4', models.IntegerField(default=0)),
                ('k5', models.SlugField(default=None, max_length=64, null=True)),
                ('v5', models.IntegerField(default=0)),
                ('k6', models.SlugField(default=None, max_length=64, null=True)),
                ('v6', models.IntegerField(default=0)),
                ('k7', models.SlugField(default=None, max_length=64, null=True)),
                ('v7', models.IntegerField(default=0)),
                ('k8', models.SlugField(default=None, max_length=64, null=True)),
                ('v8', models.IntegerField(default=0)),
                ('k9', models.SlugField(default=None, max_length=64, null=True)),
                ('v9', models.IntegerField(default=0)),
                ('k10', models.SlugField(default=None, max_length=64, null=True)),
                ('v10', models.IntegerField(default=0)),
                ('group', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='account.group')),
            ],
        ),
    ]
