# Generated by Django 4.1.3 on 2022-12-20 04:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('metrics', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='metrics',
            name='k11',
            field=models.SlugField(default=None, max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='metrics',
            name='k12',
            field=models.SlugField(default=None, max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='metrics',
            name='k13',
            field=models.SlugField(default=None, max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='metrics',
            name='k14',
            field=models.SlugField(default=None, max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='metrics',
            name='v11',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='metrics',
            name='v12',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='metrics',
            name='v13',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='metrics',
            name='v14',
            field=models.IntegerField(default=0),
        ),
    ]
