# Generated by Django 4.1.4 on 2023-11-20 03:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0009_incident_reporter_ip'),
    ]

    operations = [
        migrations.AddField(
            model_name='incident',
            name='category',
            field=models.CharField(db_index=True, default=None, max_length=124, null=True),
        ),
        migrations.AddField(
            model_name='incident',
            name='component_id',
            field=models.IntegerField(blank=True, db_index=True, default=None, null=True),
        ),
    ]
