# Generated by Django 4.1.5 on 2024-08-14 19:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('medialib', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mediaitemrendition',
            name='bytes',
            field=models.BigIntegerField(help_text='Size of rendition file'),
        ),
    ]
