# Generated by Django 4.2.11 on 2024-06-18 17:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0021_alter_cloudcredentials_group'),
    ]

    operations = [
        migrations.AlterField(
            model_name='memberdevice',
            name='modified',
            field=models.DateTimeField(auto_now=True, db_index=True),
        ),
    ]
