# Generated by Django 4.1.4 on 2024-01-11 18:34

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0018_userpasskey'),
        ('incident', '0013_rulecheck_is_required'),
    ]

    operations = [
        migrations.AddField(
            model_name='event',
            name='group',
            field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='account.group'),
        ),
        migrations.AlterField(
            model_name='rulecheck',
            name='index',
            field=models.IntegerField(db_index=True, default=0),
        ),
    ]
