# Generated by Django 4.1.4 on 2023-12-19 03:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0011_ticket'),
    ]

    operations = [
        migrations.AddField(
            model_name='rule',
            name='match_by',
            field=models.IntegerField(default=0),
        ),
    ]
