# Generated by Django 4.1.4 on 2024-02-23 19:15

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('account', '0018_userpasskey'),
        ('location', '0003_address_postalcode_suffix'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='address',
            name='modified_by',
        ),
        migrations.AddField(
            model_name='address',
            name='group',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='addresses', to='account.group'),
        ),
        migrations.AddField(
            model_name='address',
            name='label',
            field=models.CharField(blank=True, db_index=True, default=None, max_length=250, null=True),
        ),
        migrations.AddField(
            model_name='address',
            name='member',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='addresses', to=settings.AUTH_USER_MODEL),
        ),
    ]
